@@ -11,6 +11,13 @@ from .helpers import _get_instance_ids_from_names
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 def get_lab_terraform_inputs(morpheus_custom_options):
+    """
+    The get_lab_terraform_inputs function takes the provided Morpheus custom options and parses them to determine which template parameters need to be filled in. It then fills in the appropriate values for those parameters and returns a dictionary of all of the template parameters with their corresponding values.
+    
+    :param morpheus_custom_options: Pass in the custom options that have been provided by the user
+    :return: A dictionary of the key value pairs that should be used for the lab
+    :doc-author: Trelent
+    """
     # Validate the class type and import the correct file template for the template parameters to fill in
     if "administration" in morpheus_custom_options["class_type"]:
         logger.info(f'Administration class selected. Importing admin_class_config.json')
@@ -54,6 +61,17 @@ def get_lab_terraform_inputs(morpheus_custom_options):
     return(terraform_inputs)
 
 def get_terraform_layout_id(url,headers,instance_type_code):
+    """
+    The get_terraform_layout_id function takes in a url, headers and instance_type_code.
+    It then makes a request to the AS API for all of the instance type layouts that have terraform as their provisionTypeCode.
+    The function then returns the layout id of that specific layout.
+    
+    :param url: Pass in the url of the instance type
+    :param headers: Pass in the api key
+    :param instance_type_code: Get the instance type code from the instance_type_code variable
+    :return: The layout id for the terraform provision type
+    :doc-author: Trelent
+    """
     endpoint = "/api/library/instance-types"
     try:
         logger.info(f'Attempting to get the instance type with the code of: {instance_type_code}')
@@ -66,6 +84,15 @@ def get_terraform_layout_id(url,headers,instance_type_code):
     return(layout_id)
 
 def get_morpheus_terraform_plan_id(url,headers):
+    """
+    The get_morpheus_terraform_plan_id function is used to get the id of the Morpheus Terraform plan.
+    This function returns a single integer value which represents the ID of the plan.
+    
+    :param url: Specify the morpheus api url
+    :param headers: Pass the token generated by morpheus
+    :return: The id of the morpheus terraform plan
+    :doc-author: Trelent
+    """
     endpoint = "/api/service-plans?phrase=Terraform"
     try:
         logger.info(f'Attempting to get the Morpheus Terraform plan id.')
@@ -75,6 +102,23 @@ def get_morpheus_terraform_plan_id(url,headers):
     return(resp["servicePlans"][0]["id"])
 
 def get_instance_provisioning_payload(zone_id,instance_name,site_id,instance_type,instance_type_code,layout_id,plan_id,template_parameters,tags:list):
+    """
+    The get_instance_provisioning_payload function takes the provided variables and loads a template file that contains
+    the payload for the instance provisioning API call. The function then updates the template with all of the provided 
+    variables, and returns it as a JSON object.
+    
+    :param zone_id: Identify the zone in which to create the instance
+    :param instance_name: Name the instance
+    :param site_id: Determine the site to which the instance will be attached
+    :param instance_type: Determine the instance type code for the payload
+    :param instance_type_code: Determine the type of instance to be provisioned
+    :param layout_id: Specify the layout of the instance
+    :param plan_id: Determine the instance type
+    :param template_parameters: Pass in the parameters that are needed to provision an instance
+    :param tags:list: Add tags to the instance
+    :return: A payload template that can be used to provision an instance
+    :doc-author: Trelent
+    """
     try:
         logger.info(f'Attempting to load the payload template file')
         f = pkgutil.get_data(__name__, "template_files/instance_provisioning_payload.json")
@@ -97,6 +141,18 @@ def get_instance_provisioning_payload(zone_id,instance_name,site_id,instance_typ
     return(f)
 
 def set_cloud_availability(url,headers,cloud_id:int,available:bool = None):
+    """
+    The set_cloud_availability function takes a cloud_id and an availability status.
+    It then uses the location in conjunction with the Available Clouds option list to update the location to set availability.
+    
+    
+    :param url: Set the url for the api call
+    :param headers: Pass the authentication token to the api call
+    :param cloud_id:int: Identify the cloud to update
+    :param available:bool: Set the cloud to available or used
+    :return: A dictionary of the new cloud location
+    :doc-author: Trelent
+    """
     # Uses the location in conjunction with the Available Clouds option list to update the location to set availability
     try:
         logger.info(f'Attempting to get the cloud data')
@@ -120,6 +176,16 @@ def set_cloud_availability(url,headers,cloud_id:int,available:bool = None):
     return(put)
 
 def deploy_class_labs(url,headers,payloads):
+    """
+    The deploy_class_labs function takes a list of payloads and deploys them to the AS instance.
+    It returns a list of instances that were deployed.
+    
+    :param url: Specify the url of the instance endpoint
+    :param headers: Pass in the authentication token
+    :param payloads: Pass in the payloads that were generated by the generate_payloads function
+    :return: A list of instance ids
+    :doc-author: Trelent
+    """
     logger.info(f'Atttempting to provision {len(payloads)} instances')
     session = FuturesSession()
     instances = []
@@ -127,7 +193,10 @@ def deploy_class_labs(url,headers,payloads):
     futures=[session.post(f'{url}{endpoint}', headers=headers, timeout=None, verify=False, data=json.dumps(p)) for p in payloads]
     for future in as_completed(futures):
         logger.info(f'Attempting to deploy instance for student')
-        resp = future.result()
+        try:
+            resp = future.result()
+        except Exception as e:
+            logger.error(f'Something went wrong: {e}')
         if "200" in str(resp):
             logger.info("It appears the initiation of the deployment was successful")
             instance = resp.json()["instance"]["id"]
@@ -140,12 +209,23 @@ def deploy_class_labs(url,headers,payloads):
                 labs = []
                 labs.append(p["instance"]["name"])
                 instances = _get_instance_ids_from_names(url,headers,labs)
+        elif "405" in str(resp):
+            logger.error(f'Hit a 405 error. Got some bad formatting?')
         else:
             logger.error(f'An error occurred: {resp}')
-            exit
     return(instances)
 
 def lock_instance(url,headers,ids):
+    """
+    The lock_instance function takes a list of instance ids and attempts to lock them.
+    If successful, the function returns a list of locked instances.
+    
+    :param url: Pass the url of the instance to be locked
+    :param headers: Pass the authentication token to the api call
+    :param ids: Pass the instance ids to be locked
+    :return: A list of locked instances
+    :doc-author: Trelent
+    """
     locked_instances = []
     logger.info(f'Attempting to lock instances: {ids}')
     session = FuturesSession()
@@ -161,6 +241,16 @@ def lock_instance(url,headers,ids):
     return()
 
 def await_instance_deployment_status(url,headers,instance_ids):
+    """
+    The await_instance_deployment_status function is used to check the status of instances that are being provisioned.
+    It will continue to check the status until all instances have a running or failed state. It returns a dictionary with two keys: &quot;running_instances&quot; and &quot;failed_instances&quot;. The value for each key is an array of instance IDs.
+    
+    :param url: Pass the url of the instance deployment api
+    :param headers: Pass the api key to the function
+    :param instance_ids: Pass in the list of instance ids that are being checked
+    :return: A dictionary of running and failed instances
+    :doc-author: Trelent
+    """
     session = FuturesSession()
     endpoint = "/api/instances/"
     provisioning_instances = len(instance_ids)
@@ -199,6 +289,18 @@ def await_instance_deployment_status(url,headers,instance_ids):
     return(data)
 
 def tag_with_lab_ids(url,headers,target_id,instance_ids:list,tag_name):
+    """
+    The tag_with_lab_ids function takes in a target_id, instance_ids and tag name.
+    It then adds the tag to the instances with the given ids
+    
+    :param url: Specify the url of the instance
+    :param headers: Pass in the authentication token
+    :param target_id: Identify the instance that we want to add a tag to
+    :param instance_ids:list: Pass in a list of instance ids that you want to tag
+    :param tag_name: Specify the name of the tag that will be added to the instance
+    :return: Nothing
+    :doc-author: Trelent
+    """
     endpoint = "/api/instances"
     payload = {'instance':{'addTags':[{'name': tag_name,'value': instance_ids}]}}
     try:
